/*
 * Copyright (c) 2000, 2002 IBM Corp. and others..
 * All rights reserved.  This program and the accompanying materials
 * are made available under the terms of the Common Public License v0.5
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v05.html
*
 * Contributors:
 *     IBM Corporation - initial API and implementation
 */
package org.eclipse.jdt.internal.ui.dialogs;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;

import org.eclipse.jface.dialogs.IDialogSettings;
import org.eclipse.jface.dialogs.MessageDialog;

import org.eclipse.jdt.internal.ui.JavaPlugin;
import org.eclipse.jdt.internal.ui.JavaUIMessages;

/**
 * This is a <code>MessageDialog</code> which allows the user
 * to choose that the dialog isn't shown again the next time.
 */ 
public class OptionalMessageDialog extends MessageDialog {
	
	// String constants for widgets
	private static final String CHECKBOX_TEXT= JavaUIMessages.getString("OptionalMessageDialog.dontShowAgain"); //$NON-NLS-1$

	// Dialog store id constants
	private static final String STORE_ID= "OptionalMessageDialog.hide."; //$NON-NLS-1$
	
	private Button fHideDialogCheckBox;
	private String fId;
		

	/**
	 * Opens the dialog but only if the user hasn't choosen to hide it.
	 */
	public static void open(String id, Shell parent, String title, Image titleImage, String message, int dialogType, String[] buttonLabels, int defaultButtonIndex) {
		if (!isDialogEnabled(id))
			return;
		
		MessageDialog dialog= new OptionalMessageDialog(id, parent, title, titleImage, message, dialogType, buttonLabels, defaultButtonIndex);
		dialog.open();
	}

	private OptionalMessageDialog(String id, Shell parent, String title, Image titleImage, String message, int dialogType, String[] buttonLabels, int defaultButtonIndex) {
		super(parent, title, titleImage, message, dialogType, buttonLabels, defaultButtonIndex);
		fId= id;
	}

	protected Control createCustomArea(Composite parent) {
		fHideDialogCheckBox= new Button(parent, SWT.CHECK | SWT.LEFT);
		fHideDialogCheckBox.setText(CHECKBOX_TEXT);
		fHideDialogCheckBox.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				storeSetting(((Button)e.widget).getSelection());
			}
		});
		return fHideDialogCheckBox;
	}

	//--------------- Configuration handling --------------
	
	/**
	 * Returns this dialog
	 * 
	 * @return the settings to be used
	 */
	private static IDialogSettings getDialogSettings() {
		IDialogSettings settings= JavaPlugin.getDefault().getDialogSettings();
		settings= settings.getSection(STORE_ID);
		if (settings == null)
			settings= JavaPlugin.getDefault().getDialogSettings().addNewSection(STORE_ID);
		return settings;
	}
		
	/**
	 * Answers whether the optional dialog is enabled i.e. should be shown.
	 */
	static boolean isDialogEnabled(String key) {
		IDialogSettings settings= getDialogSettings();
		return !settings.getBoolean(key);
	}
	
	/**
	 * Stores the current configuration in the dialog store.
	 */
	private void storeSetting(boolean hideDialog) {
		IDialogSettings settings= getDialogSettings();
		settings.put(fId, hideDialog);
	}
}
